import sys
from PyQt5.QtWidgets import QApplication, QMainWindow
from SerialInterface import Ui_MainWindow  # Asegúrate de que el nombre del archivo y la clase coincidan
import serial
from PyQt5.QtCore import QTimer
from datetime import datetime
from PyQt5.QtGui import QIcon


class MainApp(QMainWindow, Ui_MainWindow):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.initUI()
        self.setWindowTitle("Absolutely Great Serial Terminal")
        self.setWindowIcon(QIcon('Interfaces\SerialTerminalProyect\TerminalLogo.ico'))
        self.serial_connection = None
        self.line_count = 0  

    def initUI(self):
        # Conectar botones a funciones
        self.ButtonSendData.clicked.connect(self.on_send_clicked) 
        self.ERASEDATAREC.clicked.connect(self.on_clear_clicked)
        self.ConnecButton.clicked.connect(self.on_connect_clicked)

        # Conectar botones de comandos rápidos
        self.F1.clicked.connect(lambda: self.on_fast_command_clicked("F1"))
        self.F2.clicked.connect(lambda: self.on_fast_command_clicked("F2"))
        self.F3.clicked.connect(lambda: self.on_fast_command_clicked("F3"))
        self.F1_4.clicked.connect(lambda: self.on_fast_command_clicked("F4"))
        self.F5.clicked.connect(lambda: self.on_fast_command_clicked("F5"))

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.read_from_serial)
        self.timer.start(50)  # Ajusta el intervalo en milisegundos
    
    def read_from_serial(self):
        if self.serial_connection and self.serial_connection.is_open:
            if self.serial_connection.in_waiting > 0:
                data = self.serial_connection.readline().decode().strip()
                self.update_list_widget(data)
        else:
            self.StatusLabel.setText("Serial Connection Inactive")

    def on_connect_clicked(self):
        comport = self.COMPORT.text()
        baudrate = self.COMPORT_2.text()
        if comport and baudrate.isdigit():
            try:
                if self.serial_connection:
                    self.serial_connection.close()  # Cierra la conexión existente si está abierta
                self.serial_connection = serial.Serial(comport, int(baudrate), timeout=1)
                self.StatusLabel.setText("Connected")
            except serial.SerialException as e:
                self.StatusLabel.setText("Connection Error")
        else:
            self.StatusLabel.setText("Non valid parameters.")

    def update_list_widget(self, data):
        # Aumentar el contador de línea (asumiendo que has inicializado self.line_count en __init__)
        self.line_count += 1
        
        # Obtener el timestamp actual
        timestamp = datetime.now().strftime("%H:%M:%S")
        
        # Formatear el mensaje con el número de línea, el timestamp y los datos recibidos
        message = f"{self.line_count} [{timestamp}] --> {data}"
        
        # Añadir el mensaje formateado al QListWidget
        self.listWidget.addItem(message)
        self.listWidget.scrollToBottom()  # Hace scroll al final para mostrar el último ítem añadido
    
    def send_data(self, data_to_send):
        if self.serial_connection and self.serial_connection.is_open:
            try:
                # Enviar los datos leídos por el puerto serial
                self.serial_connection.write(data_to_send.encode())  # Codificar a bytes
                self.StatusLabelSend.setText("Data Sent!")
            except serial.SerialException as e:
                self.StatusLabelSend.setText("Error Sending Data :(")
        else:
            self.StatusLabelSend.setText("Serial Connection Inactive or Closed")

    def on_send_clicked(self):
        # Leer el texto del QLineEdit DATASEND
        data_to_send = self.DATASEND.text()
        self.send_data(data_to_send)

    def on_fast_command_clicked(self, command):
        commands = {
            "F1": "ON",
            "F2": "OFF",
            "F3": "F3",
            "F4": "F4",
            "F5": "F5"
        }

        # Verificar si el comando existe en el diccionario
        if command in commands:
            data_to_send = commands[command]
            self.send_data(data_to_send)
        else:
            print("Comando no reconocido:", command)


    def on_clear_clicked(self):
        print("Clear button clicked.")
        self.listWidget.clear()  # Limpia todos los ítems del QListWidget
        self.line_count=0

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainApp()
    window.show()
    sys.exit(app.exec_())
